<?php


if ($file_name==NULL) {
	$file_name="test.kmz";
}

if ($force_scale==NULL)
    $force_scale=1;

$file_info=pathinfo($file_name);
$source_path=NULL;
$basename=$file_info['filename'];
$dest_path=$file_info['dirname']."\\";

if ($file_info['extension']=='kmz'){
	$source_path=kmz_unzip($file_name);
	$dae_file=$source_path."/models/".$basename.".dae";
}
if ($file_info['extension']=='dae')
	$dae_file=$file_name;
if ($file_info['extension']=='tri'){
	dprint("Already .tri file");
	die(0);
}
$out_file=$dest_path.$basename.".tri";

dprint("Dae file:".$dae_file);
$dom = new DomDocument;
$dom->load($dae_file);

$ftri=fopen($out_file,"w");
fprintf($ftri,"%d\n",1);


$visual_scene_url=find_one($dom,"instance_visual_scene");
$visual_scene=follow_url($visual_scene_url,find_all($dom,"visual_scene"),"visual_scene");
$nodes=$visual_scene->childNodes;
$tverts=0;
$ta=gettimeofday(True);
build_libs($lib_images,$lib_materials,$lib_effects,$lib_geometries,$lib_nodes);
set_identity($matrix);
export_nodes($nodes,$matrix);
$tb=gettimeofday(True);
$aff=sprintf("Exporting %d Verts Speed: %6.1f Verts/s",$tverts,$tverts/($tb-$ta));
dprint($aff);

kmz_unzip_remove();
fclose($ftri);

$file_name=$out_file;
dprint("Returning :".$file_name);
$file_name="aabbcc";
//die(0);

function build_libs(&$lib_images,&$lib_materials,&$lib_effects,&$lib_geometries,&$lib_nodes){
	global $dom;
	$lib=find_first($dom,"library_images");
	if ($lib) $lib_images=find_all($lib,"image");
	dprint ($lib_images->length." images");

	$lib=find_first($dom,"library_materials");
	if ($lib) $lib_materials=find_all($lib,"material");
	dprint ($lib_materials->length." materials");

	$lib=find_first($dom,"library_effects");
	if ($lib) $lib_effects=find_all($lib,"effect");
	dprint ($lib_effects->length." effects");

	$lib=find_first($dom,"library_geometries");
	if ($lib) $lib_geometries=find_all($lib,"geometry");
	dprint ($lib_geometries->length." geometries");

	$lib=find_first($dom,"library_nodes");
	if ($lib) {
		$lib_nodes=find_all($lib,"node");
		dprint ($lib_nodes->length." nodes");
	}
}

function set_identity(&$matrix){
	for($i=0;$i<4;$i++)
		for($j=0;$j<4;$j++)
			if ($i==$j)
				$matrix[$i][$j]=1;
			else
				$matrix[$i][$j]=0;
}
function mat_mul($ma,$mb){
	for ($i=0;$i<4;$i++)
		for($j=0;$j<4;$j++)
			$mr[$i][$j]=$ma[$i][0]*$mb[0][$j]+$ma[$i][1]*$mb[1][$j]+$ma[$i][2]*$mb[2][$j]+$ma[$i][3]*$mb[3][$j];
	return $mr;

}

function export_nodes($nodes,$matrixin){
	global $tverts;
	global $lib_geometries,$lib_nodes;
	foreach ($nodes as $node){
		if ($node->nodeType==XML_TEXT_NODE) continue;


		if ($node->tagName=="node") {
			//dprint ("->Entering Sub Node:".$node->getAttribute("name"));
			export_nodes($node->childNodes,$matrixin);
			$aff=sprintf("Sub Node: %s Verts:%d ",$node->getAttribute("name"),$tverts);
			dprint ($aff);
		}
		if ($node->tagName=="instance_node") {
			$nodep=$node->parentNode;
			$matrix=NULL;
			$matrix = matrix($nodep,$matrix);
			$matrix = translate($nodep,$matrix);
			$matrix = rotate($nodep,$matrix);
			$matrix = scale($nodep,$matrix);
			if($matrix==NULL)
				set_identity($matrix);
			$matrixout=mat_mul($matrixin,$matrix);
			$nnode=follow_url($node,$lib_nodes,"node" );
			export_nodes($nnode->childNodes,$matrixout);
		}
		if ($node->tagName=="instance_geometry") {
			$nodep=$node->parentNode;
			set_identity($matrix);
			$matrix = matrix($nodep,$matrix);
			$matrix = translate($nodep,$matrix);
			$matrix = rotate($nodep,$matrix);
			$matrix = scale($nodep,$matrix);
			if($matrix==NULL)
				set_identity($matrix);
			$matrixout=mat_mul($matrixin,$matrix);

			$mesh=follow_url($node,$lib_geometries,"geometry");
			$t1=gettimeofday(True);
			$nv=export_mesh($mesh,$matrixout);
			$tverts+=$nv;
			$t2=gettimeofday(True);
			$aff=sprintf("Exporting mesh:%s with %d Verts Speed: %f Verts/s",$mesh->getAttribute("name"),$nv,$nv/($t2-$t1));
			dprint($aff);
		}

	}
}
function export_mesh($mesh,$matrix){
	global $vxyz,$vxyz_offset,$vnxyz,$vnxyz_offset,$uv_array,$uv_offset,$has_uv;
	global $faces_indexes,$interleave;
	$nverts=0;
	$triangles=find_all($mesh,"triangles");
	foreach ($triangles as $triangle){
		$material_name = find_material($triangle);
		$input_vectors=find_all($triangle,"input");
		$interleave=interleave($input_vectors);
		$vxyz=$vnxyz=$uv_array=array();
		parse_inputs($mesh,$input_vectors,$vxyz,$vxyz_offset,$vnxyz,$vnxyz_offset,$uv_array,$uv_offset,$has_uv);
		$vxyz=vertices_transform($vxyz,$matrix);

		$faces_indexes=get_value($triangle,"p" );
		$faces_indexes=explode(" ",$faces_indexes);
		$n=(sizeof($faces_indexes)-1) /$interleave;
		for ($indx=0;$indx<$n;$indx++)
			write_vert($indx,$material_name);
		$nverts+=$n;

	}

	$polys=find_all($mesh,"polylist");
	foreach ($polys as $poly){
		$material_name = find_material($poly);
		$input_vectors=find_all($poly,"input");
		$interleave=interleave($input_vectors);
		$vxyz=$vnxyz=$uv_array=array();
		parse_inputs($mesh,$input_vectors,$vxyz,$vxyz_offset,$vnxyz,$vnxyz_offset,$uv_array,$uv_offset,$has_uv);
		$vxyz=vertices_transform($vxyz,$matrix);

		$faces_nvertex=get_value($poly,"vcount" );
		$faces_indexes=get_value($poly,"p" );
		$faces_nvertex=explode(" ",$faces_nvertex);
		$faces_indexes=explode(" ",$faces_indexes);

		$j=0;
		foreach($faces_nvertex as $face_n){
			if ($face_n==4) {
				$indx=array(0,1,2,2,3,0);
				for($i=0;$i<6;$i++){
					//Creating to triangle 0-1-2 and 2-3-0
					$v=$j+$indx[$i];
					write_vert($v,$material_name);
				}
				$j+=4;
				$nverts+=4;
			}
			if ($face_n==3) {
				for($i=0;$i<3;$i++)
					write_vert($j+$i,$material_name);
				$j+=3;
				$nverts+=3;
			}
		}
	}
	return $nverts;
}
function vertices_transform($vixyz,$matrix){
	for($i=0;$i<sizeof($vixyz);$i++){
		for($j=0;$j<3;$j++){
			$vxyz[$i][$j]=$matrix[$j][0]*$vixyz[$i][0]+$matrix[$j][1]*$vixyz[$i][1]+$matrix[$j][2]*$vixyz[$i][2]+$matrix[$j][3];
		}
	}
	return $vxyz;
}
function write_vert($indx,$material_name){
	global $faces_indexes,$vxyz_offset,$vnxyz_offset,$uv_offset;
	global $vxyz,$vnxyz,$uv_array,$interleave,$has_uv;
	global $ftri,$force_scale;

	$vert_indx=$faces_indexes[$indx*$interleave+$vxyz_offset];
	$vert_nindex=$faces_indexes[$indx*$interleave+$vnxyz_offset];
	$vert_uv_indx=$faces_indexes[$indx*$interleave+$uv_offset];
/*	$li=$vxyz[$vert_indx][0]*$force_scale." ".$vxyz[$vert_indx][1]*$force_scale." ".$vxyz[$vert_indx][2]*$force_scale." ";
	$li.=$vnxyz[$vert_nindex][0]." ".$vnxyz[$vert_nindex][1]." ".$vnxyz[$vert_nindex][2]." ";
	if ($has_uv)
		$li.$uv_array[$vert_uv_indx][0]." ".$uv_array[$vert_uv_indx][1]." ";
	else
		$li.="0.0 0.0 ";
	$li.=$material_name;
*/
	$li=sprintf("%9.9f %9.9f %9.9f ",$vxyz[$vert_indx][0]*$force_scale,$vxyz[$vert_indx][1]*$force_scale,$vxyz[$vert_indx][2]*$force_scale);
	$li.=sprintf("%9.9f %9.9f %9.9f ",$vnxyz[$vert_nindex][0],$vnxyz[$vert_nindex][1],$vnxyz[$vert_nindex][2]);
	if ($has_uv)
		$li.=sprintf("%6.6f %6.6f ",$uv_array[$vert_uv_indx][0],$uv_array[$vert_uv_indx][1]);
	else
		$li.=sprintf("%6.6f %6.6f ",0,0);
	$li.=$material_name;

	fprintf($ftri,$li."\n");

}
function parse_inputs($mesh,$input_vectors,&$vxyz,&$vxyz_offset,&$vnxyz,&$vnxyz_offset,&$uv_array,&$uv_offset,&$has_uv){

	$has_uv=False;
	foreach($input_vectors as $input){
		if ($input->getAttribute("semantic")=="VERTEX") {
			$vxyz_offset=$input->getAttribute("offset");

			$v1=follow_source($input,$mesh,"vertices");
			$v=find_first($v1,"input");
			$vertices_pos=follow_source($v,$mesh,"source");

			$float_array=find_first($vertices_pos,"float_array" );
			$vcount=$float_array->getAttribute("count");
			$vposvals=$float_array->nodeValue;
			$xyzpos=explode(" ",$vposvals);

			$j=0;
			for ($i=0;$i<$vcount;$i+=3){
				$vxyz[$j][0]=$xyzpos[$i];
				$vxyz[$j][1]=$xyzpos[$i+1];
				$vxyz[$j][2]=$xyzpos[$i+2];
				$j++;
			}
		}
		if ($input->getAttribute("semantic")=="NORMAL") {
			$vnxyz_offset=$input->getAttribute("offset");
			$vertices_norm=follow_source($input,$mesh,"source");

			$float_array=find_first($vertices_norm,"float_array" );
			$vcount=$float_array->getAttribute("count");
			$vnormvals=$float_array->nodeValue;
			$nxyzpos=explode(" ",$vnormvals);

			$j=0;
			for ($i=0;$i<$vcount;$i+=3){
				$vnxyz[$j][0]=$nxyzpos[$i];
				$vnxyz[$j][1]=$nxyzpos[$i+1];
				$vnxyz[$j][2]=$nxyzpos[$i+2];
				$j++;
			}

		}
		if ($input->getAttribute("semantic")=="TEXCOORD") {
			$node=follow_source($input,$mesh,"source");
			$uv_offset=$input->getAttribute("offset");

			$float_array=find_first($node,"float_array");
			$vcount=$float_array->getAttribute("count");
			$uvvals=$float_array->nodeValue;
			$uv=explode(" ",$uvvals);
			//echo "dbg:".$vcount."==".count($xyzpos)."\n<br>";
			$j=0;
			for ($i=0;$i<$vcount;$i+=2){
				$uv_array[$j][0]=$uv[$i];
				$uv_array[$j][1]=$uv[$i+1];
				$j++;
			}
			$has_uv=True;
		}
	}
}
function interleave($inputs){
	$max_offset=0;
	foreach($inputs as $input)
		$max_offset = max($max_offset,$input->getAttribute("offset"));
	return $max_offset+1;
}
function matrix($node,$matrix){

	$matrixl=get_value($node,"matrix" );
	if ($matrixl==NULL)
		return $matrix;

	$matrix2=$matrixl;
	do{
		$matrixl=$matrix2;
		$matrix2=str_replace("  "," ",$matrix2);
		$matrix2=str_replace("\n"," ",$matrix2);
		$matrix2=str_replace("\r"," ",$matrix2);
	}while($matrix2!=$matrixl);
	$matrixl=explode(' ',trim($matrix2));

	for($i=0;$i<4;$i++)
		for($j=0;$j<4;$j++)
			$matrix[$i][$j]=$matrixl[$i*4+$j];
	return $matrix;
}
function translate($node,$matrix){
	$mesh_postag=$node->getElementsByTagName("translate");
	if ($mesh_postag->length>0) {
		$mesh_pos=$mesh_postag->item(0)->nodeValue;
		$mesh_pos=explode(" ",$mesh_pos);
		for ($i=0;$i<3;$i++)
			$matrix[$i][3]=$mesh_pos[$i];
	}
	return $matrix;
}
function rotate($node,$matrix){
	$mesh_rots=$node->getElementsByTagName("rotate");
	$rx=$ry=$rz=0;
	if ($mesh_rots->length<1)
		return $matrix;
	foreach($mesh_rots as $rot){
		$axes=$rot->nodeValue;
		$axes=explode(" ",$axes);
		if ($axes[0]!=0) $rx=$axes[3]*pi()/180;
		if ($axes[1]!=0) $ry=$axes[3]*pi()/180;
		if ($axes[2]!=0) $rz=$axes[3]*pi()/180;
	}

	$sx = sin($rx);             // Setup - une seule fois
	$cx = cos($rx);
	$sy = sin($ry);
	$cy = cos($ry);
	$sz = sin($rz);
	$cz = cos($rz);
	$matrix[0][0]=$cz*$cy;   $matrix[0][1]=$cz*$sy*$sx-$sz*$cx; $matrix[0][2]=$sz*$sx+$cz*$sy*$cx;
	$matrix[1][0]=$sz*$cy;  $matrix[1][1]=$cz*$cx+$sz*$sy*$sx; $matrix[1][2]=$sz*$sy*$cx-$cz*$sx;
	$matrix[2][0]=-$sy;		 $matrix[2][1]=$cy*$sx;			$matrix[2][2]=$cy*$cx;
	return $matrix;
}
function scale($node,$matrix){
	$mesh_scaletag=$node->getElementsByTagName("scale");
	if ($mesh_scaletag->length<1)
		return $matrix;

	$mesh_scale=$mesh_scaletag->item(0)->nodeValue;
	$mesh_scale=explode(" ",$mesh_scale);
	for ($i=0;$i<3;$i++)
		for($j=0;$j<3;$j++)
			$matrix[$i][$j]=$matrix[$i][$j]*$mesh_scale[$j];
	return $matrix;

}
function get_value($node,$name){
	$noden=$node->getElementsByTagName($name);
	if ($noden->length>0) {
		return $noden->item(0)->nodeValue;
	}
	return NULL;
}
function find_all($root,$name){
	return $root->getElementsByTagName($name);
}
function find_first($root,$name){

	// root is a list
	foreach ($root as $rnode){
		$res=$rnode->getElementsByTagName($name);
		if ($res->length>0) {
			return $res->item(0);
		}
	}
	// root isn't a list
	$res=$root->getElementsByTagName($name);
	if ($res->length>0) {
		return $res->item(0);
	}

	return NULL;
}
function find_one($root,$name){
	// root is a list
	$node=NULL;
	$found=0;
	foreach ($root as $rnode){
		$res=$rnode->getElementsByTagName($name);
		if ($res->length>0) {
			if ($node==NULL) {
				$node=$res->item(0);
			}
			$found+=$res->length;
		}
	}
	if ($found==1) {
		return $node;

	}
	if ($found>1) {
		dprint("WARNING Found ".$found." ".$name." Using first one.");
		return $node;
	}
	// root isn't a list

	$res=$root->getElementsByTagName($name);
	$found=$res->length;
	if ($found==1) {
		return $res->item(0);

	}
	if ($found>1) {
		dprint("WARNING Found ".$found." ".$name." Using first one.");
		return $res->item(0);
	}
	return NULL;
}
function follow_url($element,$root,$library){
	global $dom;
	$url=$element->getAttribute("url");
	$url=split("#",$url );
	$url=$url[1];

	foreach ($root as $libelem){
		if ($libelem->getAttribute('id')==$url) {
			return $libelem;
		}
	}
	return NULL;
}
function follow_source($element,$root,$library){
	global $dom;
	$url=$element->getAttribute("source");
	$url=split("#",$url );
	$url=$url[1];

	$libs=find_all($root,$library);
	foreach ($libs as $libelem){
		if ($libelem->getAttribute('id')==$url) {
			return $libelem;
		}
	}
	return NULL;
}
function follow_material($element,$library){
	global $dom;
	$url=$element->getAttribute("material");

	$libs=find_all($dom,$library);
	foreach ($libs as $libelem){
		if ($libelem->getAttribute('symbol')==$url) {
			return $libelem;
		}
	}
	return NULL;
}
function follow_target($element,$library){
	global $dom;
	$url=$element->getAttribute("target");
	$url=split("#",$url );
	$url=$url[1];

	if (is_string($library)) {
		$libs=find_all($dom,$library);
	}else
		$libs=$library;

	foreach ($library as $libelem){
		if ($libelem->getAttribute('id')==$url) {
			return $libelem;
		}
	}
	return NULL;
}
function follow_texture($element,$library){
	global $dom;
	$url=$element->getAttribute("texture");


	$libs=find_all($dom,$library);
	foreach ($libs as $libelem){
		if ($libelem->getAttribute('sid')==$url) {
			return $libelem;
		}
	}
	return NULL;
}
function find_sid($name,$nodename){
	global $dom;
	$libs=find_all($dom,$nodename);
	foreach ($libs as $libelem){
		if ($libelem->getAttribute('sid')==$name) {
			return $libelem;
		}
	}
	return NULL;
}
function find_image($mesh,&$rgb){
	global $dom;
	global $lib_effects,$lib_materials,$lib_images;

	$rgb=False;
	$material=follow_material($mesh,"instance_material");
	if ($material==NULL) {
		$rgb=True;
		return "rgb(0.6,0.6,0.6)";
	}
	$material=follow_target($material,$lib_materials);
	$material=find_first($material,"instance_effect");
	$material=follow_url($material,$lib_effects,"effect");

	$matcolstex=find_first($material,"diffuse");
	$matcols=find_first($matcolstex,"color");
	if ($matcols!=NULL)
	{
		$matcol=$matcols->nodeValue;
		$matcol=explode(' ',$matcol);
		$rgb=True;
		return "rgb(".$matcol[0].",".$matcol[1].",".$matcol[2].")";

	}
	$tex_texture=find_first($matcolstex,"texture");
	$tex_sampler=find_sid($tex_texture->getAttribute("texture"),"newparam");
	$tex_surf=get_value($tex_sampler,"source");
	$mat_img=find_sid($tex_surf,"newparam");
	$name=get_value($mat_img,"init_from");

//	$images=find_all($lib_images,"image");
	foreach ($lib_images as $image){
		if ($image->getAttribute('id')==$name) {
			return get_value($image,"init_from");
		}
	}
}
// Find material need cleanup get_linked_element must diseappear
function find_material($mesh){
	global $source_path,$dest_path,$basename;
	global $lib_materials;

	$image=find_image($mesh,$rgb);
	if ($rgb) {
		return $image;
	}
	if ($source_path==NULL) { // No images files
		return "rgb(0.3,0.3,0.3)";
	}
	$source_file_name=$source_path."\\"."images\\".$image;
	$info=pathinfo($source_file_name);
	$dest_file_name=$dest_path.$basename."_".$info['filename'].".tga";

	if (!file_exists($dest_file_name)) {
		convtga($source_file_name,$dest_file_name);
	}

	if (file_exists($dest_file_name))
		return basename($dest_file_name);
	else
		return "rgb(0.2,0.2,0.2)";

}

function convtga($img,$image){
	$gd_info=gd_info();
	$flip=False;
	if (($gd_info==NULL)) {
		return;
	}
	dprint("Converting and scaling:".basename($img)." to ".$image);
	$ext=pathinfo($img);
	if ($ext['extension']=='jpg') {
		$ims=imagecreatefromjpeg($img);
	}
	if ($ext['extension']=='png') {
		$ims=imagecreatefrompng($img);
		$flip=True;
	}

	$sx=imagesx($ims);
	$sy=imagesy($ims);

	$psx=pow(2,floor(log($sx,2)));
	$psy=pow(2,floor(log($sy,2)));

	$imr=imagecreatetruecolor($psx,$psy);
	imagealphablending ($imr,False);

	imagecopyresampled($imr,$ims ,0,0,0,0,$psx ,$psy ,$sx ,$sy);


	$imd=fopen($image,"wb");
	$fh=sprintf("\000\000\002\000\000\000\000\000\000\000\000\000%c%c%c%c\040\000",$psx%256,floor($psx/256),$psy%256,floor($psy/256));
	fwrite($imd,$fh,18);
	for ($i=0;$i<$psy;$i++){
		for($j=0;$j<$psx;$j++){

			$rgb = imagecolorat($imr, $j, $psy-$i-1);

			$a = ($rgb >> 24) &0xFF;
			$r = ($rgb >> 16) & 0xFF;
			$g = ($rgb >> 8) & 0xFF;
			$b = $rgb & 0xFF;
			if ($a==0)
				$a=255;
			else
				$a=0;
			$d=sprintf("%c%c%c%c",$b,$g,$r,$a);
			fwrite($imd,$d,4);
		}
		//dprint($i);
	}

	fclose($imd);
}

function dprint($a){
	echo $a;
//	echo " <br>\n";

}
function xmlprint($a){
	$Body = $a;//->ownerDocument->documentElement->firstChild;
	$Document = new DOMDocument();
	$Document->appendChild($Document->importNode($Body,true));
	dprint( $Document->saveHTML());
}

function kmz_unzip($kmz_filen){

	$tmpfile = tempnam("kmz","");
	$path = dirname($tmpfile);
	unlink($tmpfile);
	dprint( "Working temp dir:".$path);
	$pathkmz=$path."\\kmz";
	// Sous php edit ne pas oublier d'ajouter l'extension zip_php dans php.ini
	$zip= new ZipArchive;
	$zip->open($kmz_filen);
	$zip->extractTo($pathkmz);
	$zip->close();

	return $pathkmz;
}
function kmz_unzip_remove(){
	$tmpfile = tempnam("kmz","");
	$path = dirname($tmpfile);
	unlink($tmpfile);
	$pathkmz=$path."\\kmz";
	recursive_remove_directory($pathkmz);
}
function recursive_remove_directory($directory, $empty=FALSE)
{
	if(substr($directory,-1) == '/')
		$directory = substr($directory,0,-1);
	if(!file_exists($directory) || !is_dir($directory))
		return FALSE;
	elseif(is_readable($directory))
	{
		$handle = opendir($directory);
		while (FALSE !== ($item = readdir($handle)))
		{
			if($item != '.' && $item != '..')
			{
				$path = $directory.'/'.$item;
				if(is_dir($path))
					recursive_remove_directory($path);
				else
					unlink($path);
			}
		}
		closedir($handle);
		if($empty == FALSE)
		{
			if(!rmdir($directory))
				return FALSE;
		}
	}
	return TRUE;
}

// Old function need to be deleted after update


?>